"""The mix-in class implementation for the CSS color setting.
"""

from apysc._color.color import Color
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._validation import arg_validation_decos


class TextFillColorCssMixIn(
    AttrLinkingMixIn,
):
    _fill_color: Color

    def _initialize_fill_color(self) -> None:
        """
        Initialize the `_fill_color` attribute.
        """
        if hasattr(self, "_fill_color"):
            return
        self._fill_color = Color("")

    @property
    def fill_color(self) -> Color:
        """
        Get a text's fill color.

        Returns
        -------
        color : Color
            A text color.

        References
        ----------
        - MultiLineText class
            - https://simon-ritchie.github.io/apysc/en/multi_line_text.html
        - Text fill_color property
            - https://simon-ritchie.github.io/apysc/en/text_fill_color.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=300,
        ...     stage_height=100,
        ...     stage_elem_id="stage",
        ... )
        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=300,
        ...     font_size=16,
        ...     x=20,
        ...     y=20,
        ... )
        >>> multi_line_text.fill_color = ap.Colors.CYAN_00AAFF
        >>> assert multi_line_text.fill_color == ap.Colors.CYAN_00AAFF
        """
        self._initialize_fill_color()
        return self._fill_color.copy()

    @fill_color.setter
    @arg_validation_decos.is_color(arg_position_index=1, optional=False)
    def fill_color(self, value: Color) -> None:
        """
        Set a text's fill color.

        Parameters
        ----------
        value : Color
            A text color.

        References
        ----------
        - MultiLineText class
            - https://simon-ritchie.github.io/apysc/en/multi_line_text.html
        - Text fill_color property
            - https://simon-ritchie.github.io/apysc/en/text_fill_color.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=300,
        ...     stage_height=100,
        ...     stage_elem_id="stage",
        ... )
        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=300,
        ...     font_size=16,
        ...     x=20,
        ...     y=20,
        ... )
        >>> multi_line_text.fill_color = ap.Colors.CYAN_00AAFF
        >>> assert multi_line_text.fill_color == ap.Colors.CYAN_00AAFF
        """
        from apysc._display.css_interface import CssInterface
        from apysc._validation import display_validation

        interface: CssInterface = display_validation.validate_css_interface(
            instance=self
        )
        self._fill_color = value
        interface.set_css(
            name="color",
            value=self._fill_color._value,
        )

        self._append_applying_new_attr_val_exp(
            new_attr=self._fill_color, attr_name="fill_color"
        )
        self._append_attr_to_linking_stack(
            attr=self._fill_color, attr_name="fill_color"
        )



"""Mix-in class implementation for the `_set_initial_border_color` method.
"""

from typing_extensions import final

from apysc._color.color import Color
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class SetInitialBorderColorMixIn:
    _border_color: Color

    @final
    @arg_validation_decos.is_color(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_border_color(
        self,
        *,
        border_color: Color,
    ) -> None:
        """
        Set a chart's initial border color.

        Parameters
        ----------
        border_color : Color
            A chart's initial border color.
        """
        self._border_color = border_color



"""The mix-in class implementation for the `_add_background` method.
"""

from typing_extensions import final

from apysc._color.color import Color
from apysc._display.rectangle import Rectangle
from apysc._display.sprite import Sprite
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class AddBackgroundMixIn:
    _background: Rectangle

    @final
    @arg_validation_decos.is_display_object_container(
        arg_position_index=1, optional=False
    )
    @arg_validation_decos.is_apysc_integer(arg_position_index=2)
    @arg_validation_decos.is_apysc_integer(arg_position_index=3)
    @arg_validation_decos.is_color(arg_position_index=4, optional=False)
    @arg_validation_decos.is_apysc_num(arg_position_index=5)
    @add_debug_info_setting(module_name=__name__)
    def _add_background(
        self,
        *,
        background_container: Sprite,
        width: Int,
        height: Int,
        background_fill_color: Color,
        background_fill_alpha: Number,
        variable_name_suffix: str,
    ) -> None:
        """
        Add a background to a container.

        Parameters
        ----------
        background_container : Sprite
            A background container instance.
        width : Int
            A background width.
        height : Int
            A background height.
        background_fill_color : Color
            A background fill-color.
        background_fill_alpha : Number
            A background fill-alpha.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        self._background = Rectangle(
            x=0,
            y=0,
            width=width,
            height=height,
            fill_color=background_fill_color,
            fill_alpha=background_fill_alpha,
            variable_name_suffix=variable_name_suffix,
        )
        background_container.add_child(self._background)



"""Mix-in class implementation for the control x path data.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class PathControlXMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
    VariableNameSuffixMixIn,
):
    _control_x: Number

    @final
    def _initialize_control_x_if_not_initialized(self) -> None:
        """
        Initialize the _control_x attribute if this instance
        does not initialize it yet.
        """
        if hasattr(self, "_control_x"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(
            value_identifier="control_x"
        )
        self._control_x = Number(
            0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_control_x_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_control_x_linking_setting(self) -> None:
        """
        Append a control_x attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._control_x, attr_name="control_x"
        )
        self._append_attr_to_linking_stack(attr=self._control_x, attr_name="control_x")

    @property
    @add_debug_info_setting(module_name=__name__)
    def control_x(self) -> Number:
        """
        Get a X-coordinate of the point.

        Returns
        -------
        control_x : Number
            X-coordinate of the control point.

        Examples
        --------
        >>> import apysc as ap
        >>> bezier_2d: ap.PathBezier2D = ap.PathBezier2D(
        ...     control_x=50, control_y=0, dest_x=100, dest_y=50
        ... )
        >>> bezier_2d.control_x = ap.Number(125)
        >>> bezier_2d.control_x
        Number(125.0)
        """
        self._initialize_control_x_if_not_initialized()
        return self._control_x.copy()

    @control_x.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def control_x(self, value: Number) -> None:
        """
        Set a X-coordinate of the control point.

        Parameters
        ----------
        value : Number
            X-coordinate of the control point.
        """
        self._initialize_control_x_if_not_initialized()
        self._control_x.value = value

        self._append_control_x_linking_setting()

    _control_x_snapshots: Optional[Dict[str, float]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_control_x_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_control_x_snapshots",
            value=int(self._control_x._value),
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._control_x._value = self._get_snapshot_val_if_exists(
            current_value=self._control_x._value,
            snapshot_dict=self._control_x_snapshots,
            snapshot_name=snapshot_name,
        )



"""The class implementation for the outlined `loop` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLoopOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `loop` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 4V1L8 5l4 4V6c3.31 0 6 2.69 6 6 0 1.01-.25 1.97-.7 2.8l1.46 1.46C19.54 15.03 20 13.57 20 12c0-4.42-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6 0-1.01.25-1.97.7-2.8L5.24 7.74C4.46 8.97 4 10.43 4 12c0 4.42 3.58 8 8 8v3l4-4-4-4v3z"/></svg>'  # noqa



"""The class implementation for the `next_plan` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialNextPlanIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `next_plan` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M12,2C6.48,2,2,6.48,2,12c0,5.52,4.48,10,10,10s10-4.48,10-10C22,6.48,17.52,2,12,2z M18,13.97h-5l2.26-2.26 c-0.91-1.06-2.25-1.74-3.76-1.74c-2.37,0-4.35,1.66-4.86,3.88l-0.96-0.32c0.64-2.62,3-4.56,5.82-4.56c1.78,0,3.37,0.79,4.47,2.03 L18,8.97V13.97z"/></g></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: assertion_basic_behavior.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# JavaScript assertion interface basic behaviors": "# JavaScript 上のアサーションのインターフェイスの基本的な挙動",  # noqa
    ##################################################
    "This page explains the JavaScript assertion interface basic behavior.": "このページではJavaScript上でのアサーション（テストなどのチェック処理）の各インターフェイスの基本的な挙動について説明します。",  # noqa
    ##################################################
    "## Interface names": "## 各インターフェイス名について",
    ##################################################
    "Each JavaScript assertion interface has the prefix of the `assert_` (e.g., `assert_equal`, `assert_true`, and so on).": "JavaScript上の各アサーションのインターフェイスは`assert_`のプレフィックスを持っています（例 : `assert_equal`や`assert_true`など）。",  # noqa
    ##################################################
    "These interfaces are positioned in the root package so you can use them, for example, `ap.assert_equal(...)`.": "これらのインターフェイスはapyscのルートのパッケージに配置されているため、たとえば`ap.assert_equal(...)`といった記述で使うことができます。",  # noqa
    ##################################################
    "## Assertion results": "## アサーションの結果について",
    ##################################################
    "These interfaces display the results on the browser console, as follows:": "これらのインターフェイスのチェック結果は以下の例のようにブラウザのコンソール上に表示されます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nint_1: ap.Int = ap.Int(10)\nap.assert_equal(left=10, right=int_1)\nap.save_overall_html(dest_dir_path="assertion_basic_behavior_results/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nint_1: ap.Int = ap.Int(10)\nap.assert_equal(left=10, right=int_1)\nap.save_overall_html(dest_dir_path="assertion_basic_behavior_results/")\n```',  # noqa
    ##################################################
    "This code displays the information message on the browser console, like this (please press the F12 key to see):": "上記のコード例ではブラウザのコンソール上に以下のような結果の情報が表示されます（ブラウザ上でF12キーを押して確認してください）:",  # noqa
    ##################################################
    "```\n[assert_equal]\nRight-side variable name: i_11\nLeft value: 10 right value: 10\n```": "```\n[assert_equal]\nRight-side variable name: i_11\nLeft value: 10 right value: 10\n```",  # noqa
    ##################################################
    "If the assertion fails, then an error message also is displayed on the browser console:": "もしチェック処理に失敗した場合も同様にブラウザのコンソール上にエラーメッセージが表示されます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nint_1: ap.Int = ap.Int(10)\nap.assert_equal(left=11, right=int_1)\nap.save_overall_html(dest_dir_path="assertion_basic_behavior_results_failed/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nint_1: ap.Int = ap.Int(10)\nap.assert_equal(left=11, right=int_1)\nap.save_overall_html(dest_dir_path="assertion_basic_behavior_results_failed/")\n```',  # noqa
    ##################################################
    "```\n[assert_equal]\nRight-side variable name: i_11\nLeft value: 11 right value: 10\n...\nAssertion failed:\n...\n```": "```\n[assert_equal]\nRight-side variable name: i_11\nLeft value: 11 right value: 10\n...\nAssertion failed:\n...\n```",  # noqa
    ##################################################
    "## Optional msg argument": "## 省略可能なmsg引数について",
    ##################################################
    "Each assertion interface has the `msg` optional argument. If you provide this argument, the error message is displayed on the browser console when an assertion fails.": "各インターフェイスは共通して`msg`という省略可能な引数のとオプションを持っています。もしこの引数に値を指定した場合、チェック処理に失敗した場合にエラーの詳細のメッセージとしてブラウザのコンソール上に表示されます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nint_1: ap.Int = ap.Int(10)\nap.assert_equal(left=11, right=int_1, msg="Values are not equal!")\nap.save_overall_html(dest_dir_path="assertion_basic_behavior_msg/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nint_1: ap.Int = ap.Int(10)\nap.assert_equal(left=11, right=int_1, msg="Values are not equal!")\nap.save_overall_html(dest_dir_path="assertion_basic_behavior_msg/")\n```',  # noqa
    ##################################################
    "```\n[assert_equal]\nRight-side variable name: i_11\nLeft value: 11 right value: 10\n...\nAssertion failed: Values are not equal!\n```": "```\n[assert_equal]\nRight-side variable name: i_11\nLeft value: 11 right value: 10\n...\nAssertion failed: Values are not equal!\n```",  # noqa
}



"""The loop implementation class for the `ap.Dictionary` keys and values.
"""

from typing import Any
from typing import Dict
from typing import Generic
from typing import Optional
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import cast

from typing_extensions import final

from apysc._expression.get_last_scope_interface import GetLastScopeInterface
from apysc._expression.indent_num import Indent
from apysc._expression.last_scope import LastScope
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.for_loop_exit_mixin import ForLoopExitMixIn
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._type.boolean import Boolean
from apysc._type.dictionary import Dictionary
from apysc._type.initialize_locals_and_globals_mixin import (
    InitializeLocalsAndGlobalsMixIn,
)
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._validation import arg_validation_decos

_DictKey = TypeVar("_DictKey", String, Int, Number, Boolean)
_DictValue = TypeVar("_DictValue", bound=InitializeWithBaseValueInterface)


class ForDictKeysAndValues(
    ForLoopExitMixIn,
    GetLastScopeInterface,
    InitializeLocalsAndGlobalsMixIn,
    Generic[_DictKey, _DictValue],
):
    """
    The loop implementation class for the `ap.Dictionary` keys and values.

    References
    ----------
    - ForDictKeysAndValues class
        - https://simon-ritchie.github.io/apysc/en/for_dict_keys_and_values.html

    Examples
    --------
    >>> import apysc as ap

    >>> _ = ap.Stage(
    ...     background_color=ap.Color("#333"), stage_width=250, stage_height=300
    ... )

    >>> dict_: ap.Dictionary[ap.Number, ap.Number] = ap.Dictionary(
    ...     {
    ...         ap.Number(50): ap.Number(50),
    ...         ap.Number(100): ap.Number(125),
    ...         ap.Number(150): ap.Number(200),
    ...     }
    ... )
    >>> with ap.ForDictKeysAndValues(
    ...     dict_=dict_,
    ...     dict_key_type=ap.Number,
    ...     dict_value_type=ap.Number,
    ... ) as (key, value):
    ...     _ = ap.Rectangle(
    ...         x=key, y=value, width=50, height=50, fill_color=ap.Color("#0af")
    ...     )
    """

    _dict: Dictionary[_DictKey, _DictValue]
    _dict_key_type: Type[_DictKey]
    _dict_value_type: Type[_DictValue]
    _variable_name_suffix: str

    @final
    @arg_validation_decos.is_apysc_dict(arg_position_index=1)
    @arg_validation_decos.is_initialize_with_base_value_interface_subclass(
        arg_position_index=2,
    )
    @arg_validation_decos.is_builtin_string(arg_position_index=6, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        dict_: Dictionary[_DictKey, _DictValue],
        dict_key_type: Type[_DictKey],
        dict_value_type: Type[_DictValue],
        *,
        locals_: Optional[Dict[str, Any]] = None,
        globals_: Optional[Dict[str, Any]] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The loop implementation class for the `ap.Dictionary` keys and values.

        Parameters
        ----------
        dict_ : Dictionary[_DictKey, _DictValue]
            A dictionary to iterate.
        dict_key_type : Type[_DictKey]
            A dictionary key type. This interface accepts hashable types,
            such as the `String`, `Int`, `Number`, or `Boolean`.
        dict_value_type : Type[_DictValue]
            A dictionary value type. This interface accepts
            `InitializeWithBaseValueInterface` subclasses,
            such as the `Int`, `String`, or `Rectangle`.
        locals_ : Optional[Dict[str, Any]], optional
            Current scope's local variables. Set locals()
            value to this argument. If specified, this interface
            reverts all local scope VariableNameMixIn
            variables (like Int, Sprite) at the end of a with-statement
            scope. This setting is useful when you don't want to
            update each variable.
        globals_ : Optional[Dict[str, Any]], optional
            Current scope's global variables. Set globals()
            value to this argument. This setting works
            the same way as the locals_ argument.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - ForDictKeysAndValues class
            - https://simon-ritchie.github.io/apysc/en/for_dict_keys_and_values.html

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage(
        ...     background_color=ap.Color("#333"), stage_width=250, stage_height=300
        ... )

        >>> dict_: ap.Dictionary[ap.Number, ap.Number] = ap.Dictionary(
        ...     {
        ...         ap.Number(50): ap.Number(50),
        ...         ap.Number(100): ap.Number(125),
        ...         ap.Number(150): ap.Number(200),
        ...     }
        ... )
        >>> with ap.ForDictKeysAndValues(
        ...     dict_=dict_,
        ...     dict_key_type=ap.Number,
        ...     dict_value_type=ap.Number,
        ... ) as (key, value):
        ...     _ = ap.Rectangle(
        ...         x=key, y=value, width=50, height=50, fill_color=ap.Color("#0af")
        ...     )
        """
        self._initialize_locals_and_globals(locals_, globals_=globals_)
        self._dict = dict_
        self._dict_key_type = dict_key_type
        self._dict_value_type = dict_value_type
        self._variable_name_suffix = variable_name_suffix
        self._indent = Indent()

    @final
    def _get_last_scope(self) -> LastScope:
        """
        Get a target last scope value.

        Returns
        -------
        last_scope : LastScope
            A target last scope.
        """
        return LastScope.FOR_DICT_KEYS_AND_VALUES

    @final
    @add_debug_info_setting(module_name=__name__)
    def __enter__(self) -> Tuple[_DictKey, _DictValue]:
        """
        The entering method for the beginning of with-statement.

        Returns
        -------
        dict_key : _DictKey
            A dictionary key of iteration.
        dict_value : _DictValue
            A dictionary value of iteration.
        """
        from apysc._expression import expression_data_util
        from apysc._loop import loop_count
        from apysc._type import revert_mixin
        from apysc._validation.variable_name_validation import (
            validate_variable_name_mixin_type,
        )

        loop_count.increment_current_loop_count()
        self._snapshot_name = revert_mixin.make_snapshots_of_each_scope_vars(
            locals_=self._locals, globals_=self._globals
        )
        dict_key: _DictKey = cast(
            _DictKey,
            self._dict_key_type._initialize_with_base_value(),
        )
        dict_value: _DictValue = self._dict_value_type._initialize_with_base_value()
        dict_value_variable_name: str = validate_variable_name_mixin_type(
            instance=dict_value,
        ).variable_name

        expression: str = (
            f"for ({dict_key.variable_name} in {self._dict.variable_name}) {{"
            f"\n  {dict_value_variable_name} = {self._dict.variable_name}"
            f"[{dict_key.variable_name}];"
        )
        expression_data_util.append_js_expression(expression=expression)

        self._indent.__enter__()
        return dict_key, dict_value



"""The class implementation for the `filter_list_alt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFilterListAltIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `filter_list_alt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M0 0h24m0 24H0" fill="none"/><path d="M4.25 5.66c.1.13 5.74 7.33 5.74 7.33V19c0 .55.45 1 1.01 1h2.01c.55 0 1.01-.45 1.01-1v-6.02s5.49-7.02 5.75-7.34C20.03 5.32 20 5 20 5c0-.55-.45-1-1.01-1H5.01C4.4 4 4 4.48 4 5c0 .2.06.44.25.66z"/></svg>'  # noqa



"""Dash setting class implementation for a line.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_to_apysc_val_from_builtin_mixin import (
    AttrToApyscValFromBuiltinMixIn,
)
from apysc._type.dictionary import Dictionary
from apysc._type.int import Int
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class LineDashSetting(
    Dictionary[str, Int],
    VariableNameSuffixAttrOrVarMixIn,
    AttrToApyscValFromBuiltinMixIn,
):
    """
    Dash setting class for a line.

    References
    ----------
    - Graphics line_style interface
        - https://simon-ritchie.github.io/apysc/en/graphics_line_style.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)
    >>> line: ap.Line = sprite.graphics.draw_line(
    ...     x_start=50, y_start=50, x_end=150, y_end=50
    ... )
    >>> line.line_dash_setting = ap.LineDashSetting(dash_size=5, space_size=2)
    >>> line.line_dash_setting.dash_size
    Int(5)

    >>> line.line_dash_setting.space_size
    Int(2)
    """

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=1, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=2, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=2, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=3, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        dash_size: Union[int, Int],
        space_size: Union[int, Int],
        variable_name_suffix: str = ""
    ) -> None:
        """
        Dash setting class for a line.

        Parameters
        ----------
        dash_size : int or Int
            Dash size.
        space_size : int or Int
            Blank space size between dashes.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - Graphics line_style interface
            - https://simon-ritchie.github.io/apysc/en/graphics_line_style.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50
        ... )
        >>> line.line_dash_setting = ap.LineDashSetting(dash_size=5, space_size=2)
        >>> line.line_dash_setting.dash_size
        Int(5)

        >>> line.line_dash_setting.space_size
        Int(2)
        """
        self._variable_name_suffix = variable_name_suffix
        dash_size_: Int = self._get_copied_int_from_builtin_val(
            integer=dash_size, attr_identifier="dash_size"
        )
        space_size_: Int = self._get_copied_int_from_builtin_val(
            integer=space_size, attr_identifier="space_size"
        )
        super(LineDashSetting, self).__init__(
            {
                "dash_size": dash_size_,
                "space_size": space_size_,
            },
            variable_name_suffix=self._variable_name_suffix,
        )

    @property
    @add_debug_info_setting(module_name=__name__)
    def dash_size(self) -> Int:
        """
        Get a dash size setting.

        Returns
        -------
        dash_size : Int
            Dash size setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50
        ... )
        >>> line.line_dash_setting = ap.LineDashSetting(dash_size=5, space_size=2)
        >>> line.line_dash_setting.dash_size
        Int(5)
        """
        return self["dash_size"]

    @property
    @add_debug_info_setting(module_name=__name__)
    def space_size(self) -> Int:
        """
        Get a space size setting.

        Returns
        -------
        space_size : Int
            Space size setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50
        ... )
        >>> line.line_dash_setting = ap.LineDashSetting(dash_size=5, space_size=2)
        >>> line.line_dash_setting.space_size
        Int(2)
        """
        return self["space_size"]



"""The rectangle's geometry class implementation.
"""

from typing_extensions import final

from apysc._geom.rectangle_geom_bottom_y_mixin import RectangleGeomBottomYMixIn
from apysc._geom.rectangle_geom_center_x_mixin import RectangleGeomCenterXMixIn
from apysc._geom.rectangle_geom_center_y_mixin import RectangleGeomCenterYMixIn
from apysc._geom.rectangle_geom_height_mixin import RectangleGeomHeightMixIn
from apysc._geom.rectangle_geom_left_x_mixin import RectangleGeomLeftXMixIn
from apysc._geom.rectangle_geom_right_x_mixin import RectangleGeomRightXMixIn
from apysc._geom.rectangle_geom_top_y_mixin import RectangleGeomTopYMixIn
from apysc._geom.rectangle_geom_width_mixin import RectangleGeomWidthMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class RectangleGeom(
    VariableNameSuffixAttrOrVarMixIn,
    VariableNameMixIn,
    VariableNameSuffixMixIn,
    RectangleGeomLeftXMixIn,
    RectangleGeomCenterXMixIn,
    RectangleGeomRightXMixIn,
    RectangleGeomTopYMixIn,
    RectangleGeomCenterYMixIn,
    RectangleGeomBottomYMixIn,
    RectangleGeomWidthMixIn,
    RectangleGeomHeightMixIn,
):
    """
    The rectangle's geometry class.

    Attributes
    ----------
    left_x : Number
        The rectangle left x coordinate.
    center_x : Number
        The rectangle center x coordinate.
    right_x : Number
        The rectangle right x coordinate.
    top_y : Number
        The rectangle top y coordinate.
    center_y : Number
        The rectangle center y coordinate.
    bottom_y : Number
        The rectangle bottom y coordinate.
    width : Int
        The rectangle width.
    height : Int
        The Rectangle height.

    References
    ----------
    - RectangleGeom class
        - https://simon-ritchie.github.io/apysc/en/rectangle_geom.html
    """

    @final
    # left_x
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    # center_x
    @arg_validation_decos.is_apysc_num(arg_position_index=2)
    # right_x
    @arg_validation_decos.is_apysc_num(arg_position_index=3)
    # top_y
    @arg_validation_decos.is_apysc_num(arg_position_index=4)
    # center_y
    @arg_validation_decos.is_apysc_num(arg_position_index=5)
    # bottom_y
    @arg_validation_decos.is_apysc_num(arg_position_index=6)
    # width
    @arg_validation_decos.is_apysc_num(arg_position_index=7)
    # height
    @arg_validation_decos.is_apysc_num(arg_position_index=8)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        left_x: Number,
        center_x: Number,
        right_x: Number,
        top_y: Number,
        center_y: Number,
        bottom_y: Number,
        width: Int,
        height: Int,
    ):
        """
        The rectangle's geometry class.

        Parameters
        ----------
        left_x : Number
            The rectangle left x coordinate.
        center_x : Number
            The rectangle center x coordinate.
        right_x : Number
            The rectangle right x coordinate.
        top_y : Number
            The rectangle top y coordinate.
        center_y : Number
            The rectangle center y coordinate.
        bottom_y : Number
            The rectangle bottom y coordinate.
        width : Int
            The rectangle width.
        height : Int
            The Rectangle height.

        References
        ----------
        - RectangleGeom class
            - https://simon-ritchie.github.io/apysc/en/rectangle_geom.html
        """
        self._left_x = left_x
        self._center_x = center_x
        self._right_x = right_x
        self._top_y = top_y
        self._center_y = center_y
        self._bottom_y = bottom_y
        self._width = width
        self._height = height



"""Mix-in class implementation for the `_set_initial_background_fill_alpha`
method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class SetInitialBackgroundFillAlphaMixIn:
    _background_fill_alpha: Number

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_background_fill_alpha(
        self,
        *,
        background_fill_alpha: Union[float, Number],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set a chart's initial background fill-alpha.

        Parameters
        ----------
        background_fill_alpha : Union[float, Number]
            A chart's initial background fill-alpha.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(background_fill_alpha, Number):
            background_fill_alpha_: Number = Number(
                background_fill_alpha,
                variable_name_suffix=variable_name_suffix,
            )
        else:
            background_fill_alpha_ = background_fill_alpha
        self._background_fill_alpha = background_fill_alpha_



"""The class implementation for the outlined `highlight_alt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialHighlightAltOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `highlight_alt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M17,5h-2V3h2V5z M15,15v6l2.29-2.29L19.59,21L21,19.59l-2.29-2.29L21,15H15z M19,9h2V7h-2V9z M19,13h2v-2h-2V13z M11,21h2 v-2h-2V21z M7,5h2V3H7V5z M3,17h2v-2H3V17z M5,21v-2H3C3,20.1,3.9,21,5,21z M19,3v2h2C21,3.9,20.1,3,19,3z M11,5h2V3h-2V5z M3,9h2 V7H3V9z M7,21h2v-2H7V21z M3,13h2v-2H3V13z M3,5h2V3C3.9,3,3,3.9,3,5z"/></g></svg>'  # noqa



"""Class implementation for the polygon-related classes' x3 mix-in.
"""

from apysc._display.polygon_apply_current_points_mixin import (
    PolygonApplyCurrentPointsMixIn,
)
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class PolygonX3MixIn(PolygonApplyCurrentPointsMixIn):
    _x3: Number

    @property
    @add_debug_info_setting(module_name=__name__)
    def x3(self) -> Number:
        """
        Get a third x-coordinate.

        Returns
        -------
        x3 : Number
            A third x-coordinate.
        """
        x3: Number = self._points[2].x
        return x3

    @x3.setter
    @add_debug_info_setting(module_name=__name__)
    def x3(self, value: Number) -> None:
        """
        Set a third x-coordinate.

        Parameters
        ----------
        value : Number
            Y-coordinate to set.
        """
        from apysc._geom.point2d import Point2D

        self._x3._value = value._value
        new_point: Point2D = self._points[2]
        new_point.x = value
        self._points[2] = new_point
        self._apply_current_points()



"""Class implementations for now-related mix-in.
"""

from datetime import datetime
from typing import TYPE_CHECKING

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._time.datetime_ import DateTime


class NowMixIn:
    @classmethod
    @final
    @add_debug_info_setting(module_name=__name__)
    def now(cls, *, variable_name_suffix: str = "") -> "DateTime":
        """
        Get a `DateTime` instance of the current time.

        Parameters
        ----------
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        dt : DateTime
            A created `DateTime` instance.

        References
        ----------
        - DateTime class now interface
            - https://simon-ritchie.github.io/apysc/en/datetime_now.html

        Examples
        --------
        >>> from datetime import datetime
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> py_now: datetime = datetime.now()
        >>> ap_now: ap.DateTime = ap.DateTime.now()
        >>> ap_now.year == py_now.year
        Boolean(True)
        >>> ap_now.month == py_now.month
        Boolean(True)
        >>> ap_now.day == py_now.day
        Boolean(True)
        """
        from apysc._time.datetime_ import DateTime

        now_: datetime = datetime.now()
        dt: DateTime = DateTime(
            year=1970,
            month=1,
            day=1,
            variable_name_suffix=variable_name_suffix,
        )
        dt._year._value = now_.year
        dt._month._value = now_.month
        dt._day._value = now_.day
        dt._hour._value = now_.hour
        dt._minute._value = now_.minute
        dt._second._value = now_.second
        _append_now_expression(dt=dt)
        return dt


@add_debug_info_setting(module_name=__name__)
@arg_validation_decos.is_apysc_datetime(arg_position_index=0)
def _append_now_expression(*, dt: "DateTime") -> None:
    """
    Append a `now` interface expression string.

    Parameters
    ----------
    dt : DateTime
        A target `DateTime` instance.
    """
    from apysc._expression import expression_data_util

    expression: str = f"{dt.variable_name} = new Date();"
    expression_data_util.append_js_expression(expression=expression)



"""The class implementation for the outlined `settings_bluetooth` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSettingsBluetoothOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `settings_bluetooth` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M11 24h2v-2h-2v2zm-4 0h2v-2H7v2zm8 0h2v-2h-2v2zm2.71-18.29L12 0h-1v7.59L6.41 3 5 4.41 10.59 10 5 15.59 6.41 17 11 12.41V20h1l5.71-5.71-4.3-4.29 4.3-4.29zM13 3.83l1.88 1.88L13 7.59V3.83zm1.88 10.46L13 16.17v-3.76l1.88 1.88z"/></svg>'  # noqa



"""Mix-in Class implementation for the `_set_initial_width` method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class SetInitialWidthMixIn:
    _width: Int

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_width(
        self,
        *,
        width: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set a chart's initial width.

        Parameters
        ----------
        width : Union[int, Int]
            A chart's initial width.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(width, Int):
            width_: Int = Int(width, variable_name_suffix=variable_name_suffix)
        else:
            width_ = width
        self._width = width_



"""The class implementation for the `find_replace` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFindReplaceIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `find_replace` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M11 6c1.38 0 2.63.56 3.54 1.46L12 10h6V4l-2.05 2.05C14.68 4.78 12.93 4 11 4c-3.53 0-6.43 2.61-6.92 6H6.1c.46-2.28 2.48-4 4.9-4zm5.64 9.14c.66-.9 1.12-1.97 1.28-3.14H15.9c-.46 2.28-2.48 4-4.9 4-1.38 0-2.63-.56-3.54-1.46L10 12H4v6l2.05-2.05C7.32 17.22 9.07 18 11 18c1.55 0 2.98-.51 4.14-1.36L20 21.49 21.49 20l-4.85-4.86z"/></svg>'  # noqa



"""The class implementation for the outlined `request_page` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRequestPageOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `request_page` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M13.17,4L18,8.83V20H6V4H13.17 M14,2H6C4.9,2,4,2.9,4,4v16c0,1.1,0.9,2,2,2h12c1.1,0,2-0.9,2-2V8L14,2L14,2z M15,11h-4v1h3 c0.55,0,1,0.45,1,1v3c0,0.55-0.45,1-1,1h-1v1h-2v-1H9v-2h4v-1h-3c-0.55,0-1-0.45-1-1v-3c0-0.55,0.45-1,1-1h1V8h2v1h2V11z"/></svg>'  # noqa



"""The class implementation for the outlined `assignment` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAssignmentOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `assignment` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M7 15h7v2H7zm0-4h10v2H7zm0-4h10v2H7zm12-4h-4.18C14.4 1.84 13.3 1 12 1c-1.3 0-2.4.84-2.82 2H5c-.14 0-.27.01-.4.04-.39.08-.74.28-1.01.55-.18.18-.33.4-.43.64-.1.23-.16.49-.16.77v14c0 .27.06.54.16.78s.25.45.43.64c.27.27.62.47 1.01.55.13.02.26.03.4.03h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-7-.25c.41 0 .75.34.75.75s-.34.75-.75.75-.75-.34-.75-.75.34-.75.75-.75zM19 19H5V5h14v14z"/></svg>'  # noqa



"""Definition of the FPS enum.
"""

from enum import Enum
from typing import Union

from typing_extensions import final


class FPSDefinition:
    _fps: int
    _millisecond_interval: Union[int, float]

    @final
    def __init__(self, *, fps: int, millisecond_interval: Union[int, float]) -> None:
        """
        FPS definition class.

        Parameters
        ----------
        fps : int
            FPS values, such as 30 and 60.
        millisecond_interval : int or float
            FPS value in millisecond intervals, such as 33.333...
        """
        self._fps = fps
        self._millisecond_interval = millisecond_interval

    @property
    def millisecond_interval(self) -> float:
        """
        Get a FPS value in milisecond intervals.

        Returns
        -------
        milisecond_interval : float
            FPS value in milisecond intervals, such as 33.333...
        """
        return self._millisecond_interval


class FPS(Enum):
    """
    Definition of the FPS enum.

    References
    ----------
    - FPS enum
        - https://simon-ritchie.github.io/apysc/en/fps.html

    Examples
    --------
    >>> import apysc as ap
    >>> _ = ap.Stage()
    >>> def on_timer(e: ap.TimerEvent, options: dict) -> None:
    ...     pass
    >>> ap.Timer(on_timer, delay=ap.FPS.FPS_60).start()
    """

    FPS_5 = FPSDefinition(fps=5, millisecond_interval=200)
    FPS_10 = FPSDefinition(fps=10, millisecond_interval=100)
    FPS_15 = FPSDefinition(fps=15, millisecond_interval=66.6666667)
    FPS_20 = FPSDefinition(fps=20, millisecond_interval=50)
    FPS_25 = FPSDefinition(fps=25, millisecond_interval=40)
    FPS_30 = FPSDefinition(fps=30, millisecond_interval=33.3333333)
    FPS_35 = FPSDefinition(fps=35, millisecond_interval=28.5714286)
    FPS_40 = FPSDefinition(fps=40, millisecond_interval=25)
    FPS_45 = FPSDefinition(fps=45, millisecond_interval=22.2222222)
    FPS_50 = FPSDefinition(fps=50, millisecond_interval=20)
    FPS_55 = FPSDefinition(fps=55, millisecond_interval=18.1818182)
    FPS_60 = FPSDefinition(fps=60, millisecond_interval=16.6666667)



"""The mix-in class implementation for the `is_display_axis_label` value.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._validation import arg_validation_decos


class IsDisplayAxisLabelMixIn:
    _is_display_axis_label: Boolean

    @final
    @arg_validation_decos.is_boolean(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_is_display_axis_label(
        self,
        *,
        is_display_axis_label: Union[bool, Boolean],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set an initial boolean, whether an axis label is visible or not.

        Parameters
        ----------
        is_display_axis_label : Union[bool, Boolean]
            A boolean, whether an axis label is visible or not.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(is_display_axis_label, Boolean):
            is_display_axis_label_: Boolean = Boolean(
                is_display_axis_label,
                variable_name_suffix=variable_name_suffix,
            )
        else:
            is_display_axis_label_ = is_display_axis_label
        self._is_display_axis_label = is_display_axis_label_



"""Class implementations for the total seconds-related mix-in.
"""

from datetime import datetime
from datetime import timedelta
from typing import TYPE_CHECKING

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._time.left_and_right_datetimes_mixin import LeftAndRightDatetimesMixIn
from apysc._type.number import Number
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._time.datetime_ import DateTime


class TotalSecondsMixIn(
    VariableNameMixIn,
    VariableNameSuffixAttrOrVarMixIn,
    LeftAndRightDatetimesMixIn,
):
    _total_seconds_value: float

    @final
    @arg_validation_decos.is_apysc_datetime(arg_position_index=1)
    @arg_validation_decos.is_apysc_datetime(arg_position_index=2)
    @add_debug_info_setting(module_name=__name__)
    def _set_init_total_seconds_value_for_python(
        self,
        *,
        left_datetime: "DateTime",
        right_datetime: "DateTime",
    ) -> None:
        """
        Set an initial total seconds value for Python.

        Parameters
        ----------
        left_datetime : DateTime
            A left-side `DateTime` instance to compare.
        right_datetime : DateTime
            A right-side `DateTime` instance to compare.
        """
        left_py_datetime: datetime
        right_py_datetime: datetime
        (
            left_py_datetime,
            right_py_datetime,
        ) = self._get_left_and_right_py_datetimes_from_apysc_datetime(
            left_apysc_datetime=left_datetime,
            right_apysc_datetime=right_datetime,
        )
        timedelta_: timedelta = left_py_datetime - right_py_datetime
        self._total_seconds_value = timedelta_.total_seconds()

    @final
    @add_debug_info_setting(module_name=__name__)
    def total_seconds(self) -> Number:
        """
        Get the total seconds in the duration.

        Returns
        -------
        total_seconds : Number
            Total seconds in the duration.

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> datetime_1: ap.DateTime = ap.DateTime(2022, 12, 7)
        >>> datetime_2: ap.DateTime = ap.DateTime(2022, 12, 6)
        >>> timedelta_: ap.TimeDelta = datetime_1 - datetime_2
        >>> timedelta_.total_seconds()
        Number(86400.0)

        References
        ----------
        - TimeDelta class
            - https://simon-ritchie.github.io/apysc/en/timedelta.html
        - TimeDelta class total_seconds interface
            - https://simon-ritchie.github.io/apysc/en/timedelta_total_seconds.html
        """
        suffix: str = self._get_attr_or_variable_name_suffix(
            value_identifier="total_seconds"
        )
        total_seconds: Number = Number(
            0,
            variable_name_suffix=suffix,
        )
        total_seconds._value = self._total_seconds_value
        self._append_total_seconds_expression(total_seconds=total_seconds)
        return total_seconds

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_total_seconds_expression(self, *, total_seconds: Number) -> None:
        """
        Append a total seconds' expression string.

        Parameters
        ----------
        total_seconds : Number
            Total seconds value.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{total_seconds.variable_name} = {self.variable_name} / 1000;"
        )
        expression_data_util.append_js_expression(expression=expression)



"""The class implementation for the outlined `account_box` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAccountBoxOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `account_box` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19 5v14H5V5h14m0-2H5c-1.11 0-2 .9-2 2v14c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-7 9c-1.65 0-3-1.35-3-3s1.35-3 3-3 3 1.35 3 3-1.35 3-3 3zm0-4c-.55 0-1 .45-1 1s.45 1 1 1 1-.45 1-1-.45-1-1-1zm6 10H6v-1.53c0-2.5 3.97-3.58 6-3.58s6 1.08 6 3.58V18zm-9.69-2h7.38c-.69-.56-2.38-1.12-3.69-1.12s-3.01.56-3.69 1.12z"/></svg>'  # noqa



"""The class implementation for the outlined `flight_takeoff` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFlightTakeoffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `flight_takeoff` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M2.5 19h19v2h-19v-2zm19.57-9.36c-.21-.8-1.04-1.28-1.84-1.06L14.92 10l-6.9-6.43-1.93.51 4.14 7.17-4.97 1.33-1.97-1.54-1.45.39 2.59 4.49L21 11.49c.81-.23 1.28-1.05 1.07-1.85z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: material_icon.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Material icon": "# マテリアルアイコン",
    ##################################################
    "This page explains the material icon's apysc implementations.": "このページではapyscにおけるマテリアルアイコンの実装について説明します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each material icon exists in the root package (e.g., `ap.MaterialTimelineIcon`).": "各マテリアルアイコンは一番上のパッケージパスに存在します（例 : `ap.MaterialTimelineIcon`）。",  # noqa
    ##################################################
    "Also, each material icon name has the prefix of `Material` and suffix of `Icon`.": "また、各マテリアルアイコンの名前は`Material`というプレフィックスと`Icon`というサフィックスを共通で持ちます。",  # noqa
    ##################################################
    "All material icons' constructor has the coordinates and style settings arguments, such as the `x`, `y`, `size`, `fill_color`, and `fill_alpha`.": "全てのマテリアルアイコンのコンストラクタは`x`や`y`、`size`、`fill_color`、`fill_alpha`などの座標やスタイルの設定の引数を持ちます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=168,\n    stage_height=72,\n    stage_elem_id="stage",\n)\nSIZE: int = 24\n\nap.MaterialHomeIcon(\n    x=24,\n    y=24,\n    size=SIZE,\n    fill_color=ap.Colors.CYAN_00AAFF,\n)\nap.MaterialBuildIcon(\n    x=24 * 3,\n    y=24,\n    size=SIZE,\n    fill_color=ap.Colors.CYAN_00AAFF,\n)\nap.MaterialCheckCircleIcon(\n    x=24 * 5,\n    y=24,\n    size=SIZE,\n    fill_color=ap.Colors.CYAN_00AAFF,\n)\n\nap.save_overall_html(dest_dir_path="material_icon_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=168,\n    stage_height=72,\n    stage_elem_id="stage",\n)\nSIZE: int = 24\n\nap.MaterialHomeIcon(\n    x=24,\n    y=24,\n    size=SIZE,\n    fill_color=ap.Colors.CYAN_00AAFF,\n)\nap.MaterialBuildIcon(\n    x=24 * 3,\n    y=24,\n    size=SIZE,\n    fill_color=ap.Colors.CYAN_00AAFF,\n)\nap.MaterialCheckCircleIcon(\n    x=24 * 5,\n    y=24,\n    size=SIZE,\n    fill_color=ap.Colors.CYAN_00AAFF,\n)\n\nap.save_overall_html(dest_dir_path="material_icon_basic_usage_1/")\n```',  # noqa
    ##################################################
    "You can also set coordinates or styles with an instance's attributes as follows:": "以下のようにインスタンスの属性を用いて座標やスタイルを設定することもできます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=168,\n    stage_height=72,\n    stage_elem_id="stage",\n)\nSIZE: int = 24\n\nhome_icon: ap.MaterialHomeIcon = ap.MaterialHomeIcon(size=SIZE)\nhome_icon.x = ap.Number(24)\nhome_icon.y = ap.Number(24)\nhome_icon.fill_color = ap.Colors.CYAN_00AAFF\n\nbuild_icon: ap.MaterialBuildIcon = ap.MaterialBuildIcon(size=SIZE)\nbuild_icon.x = ap.Number(24 * 3)\nbuild_icon.y = ap.Number(24)\nbuild_icon.fill_color = ap.Colors.CYAN_00AAFF\n\ncheck_circle_icon: ap.MaterialCheckCircleIcon = ap.MaterialCheckCircleIcon(\n    size=SIZE,\n)\ncheck_circle_icon.x = ap.Number(24 * 5)\ncheck_circle_icon.y = ap.Number(24)\ncheck_circle_icon.fill_color = ap.Colors.CYAN_00AAFF\n\nap.save_overall_html(dest_dir_path="material_icon_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=168,\n    stage_height=72,\n    stage_elem_id="stage",\n)\nSIZE: int = 24\n\nhome_icon: ap.MaterialHomeIcon = ap.MaterialHomeIcon(size=SIZE)\nhome_icon.x = ap.Number(24)\nhome_icon.y = ap.Number(24)\nhome_icon.fill_color = ap.Colors.CYAN_00AAFF\n\nbuild_icon: ap.MaterialBuildIcon = ap.MaterialBuildIcon(size=SIZE)\nbuild_icon.x = ap.Number(24 * 3)\nbuild_icon.y = ap.Number(24)\nbuild_icon.fill_color = ap.Colors.CYAN_00AAFF\n\ncheck_circle_icon: ap.MaterialCheckCircleIcon = ap.MaterialCheckCircleIcon(\n    size=SIZE,\n)\ncheck_circle_icon.x = ap.Number(24 * 5)\ncheck_circle_icon.y = ap.Number(24)\ncheck_circle_icon.fill_color = ap.Colors.CYAN_00AAFF\n\nap.save_overall_html(dest_dir_path="material_icon_basic_usage_2/")\n```',  # noqa
    ##################################################
    "## Material icon's license": "## マテリアルアイコンのライセンス",
    ##################################################
    "The apysc library uses material icons licensed under the APACHE LICENSE, VERSION 2.0.": "apyscライブラリはAPACHE LICENSE, VERSION 2.0のライセンスのマテリアルアイコンを使用しています。",  # noqa
    ##################################################
    "- [Material Symbols & Icons](https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed)": "- [Material Symbols & Icons](https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed)",  # noqa
    ##################################################
    "- [material-design-icons (GitHub)](https://github.com/google/material-design-icons)": "- [material-design-icons (GitHub)](https://github.com/google/material-design-icons)",  # noqa
    ##################################################
    "- [APACHE LICENSE, VERSION 2.0](https://www.apache.org/licenses/LICENSE-2.0.html)": "- [APACHE LICENSE, VERSION 2.0](https://www.apache.org/licenses/jp_LICENSE-2.0.html)",  # noqa
    ##################################################
    "## Each material icon constructor API": "## 各マテリアルアイコンのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "The class implementation for the SVG icon's class.<hr>": "SVGアイコンのためのクラスの実装。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x`: Union[float, Number], optional": "- `x`: Union[float, Number], optional",
    ##################################################
    "  - X-coordinate of the icon.": "  - アイコンのX座標。",
    ##################################################
    "- `y`: Union[float, Number], optional": "- `y`: Union[float, Number], optional",
    ##################################################
    "  - Y-coordinate of the icon.": "  - アイコンのY座標。",
    ##################################################
    "- `size`: Union[int, Int], optional": "- `size`: Union[int, Int], optional",
    ##################################################
    "  - Size of the icon.": "  - アイコンのY座標。",
    ##################################################
    "- `fill_color`: Color, optional": "- `fill_color`: Color, optional",
    ##################################################
    "  - Fill-color of the icon.": "  - アイコンの塗りの色。",
    ##################################################
    "- `fill_alpha`: Union[float, Number], optional": "- `fill_alpha`: Union[float, Number], optional",  # noqa
    ##################################################
    "  - Fill-alpha of the icon.": "  - アイコンの塗りの透明度。",
    ##################################################
    "- `parent`: Optional[ChildMixIn], optional": "- `parent`: Optional[ChildMixIn], optional",  # noqa
    ##################################################
    "  - A parent instance to add this instance. If the specified value is None, this interface uses a stage instance.": "  - このインスタンスを追加する親のインスタンス。もしもNoneが指定された場合、このインスタンスはステージのインスタンスへと追加されます。",  # noqa
    ##################################################
    "- `variable_name_suffix`: str, optional": "- `variable_name_suffix`: str, optional",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
}



"""The mix-in class implementation for the `_initialize_each_container` method.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class InitializeEachContainerMixIn:
    @final
    @arg_validation_decos.is_builtin_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _initialize_each_container(self, *, variable_name_suffix: str) -> None:
        """
        Initialize each container instance.

        Parameters
        ----------
        variable_name_suffix : str
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Raises
        ------
        TypeError
            If this instance is not an instance of `OverallContainerMixIn`.
        """
        from apysc._chart.background_container_mixin import BackgroundContainerMixIn
        from apysc._chart.border_container_mixin import BorderContainerMixIn
        from apysc._chart.chart_container_mixin import ChartContainerMixIn
        from apysc._chart.overall_container_mixin import OverallContainerMixIn
        from apysc._chart.x_axis_container_mixin import XAxisContainerMixIn
        from apysc._chart.y_axis_container_mixin import YAxisContainerMixIn
        from apysc._display.sprite import Sprite

        if not isinstance(self, OverallContainerMixIn):
            raise TypeError(
                "This interface only supports an `OverallContainerMixIn` "
                f"instance: {type(self).__name__}, {self}"
            )
        self._initialize_overall_container(variable_name_suffix=variable_name_suffix)
        overall_container: Sprite = self._overall_container

        if isinstance(self, BackgroundContainerMixIn):
            self._initialize_background_container(
                overall_container=overall_container,
                variable_name_suffix=variable_name_suffix,
            )
        if isinstance(self, ChartContainerMixIn):
            self._initialize_chart_container(
                overall_container=overall_container,
                variable_name_suffix=variable_name_suffix,
            )
        if isinstance(self, XAxisContainerMixIn):
            self._initialize_x_axis_container(
                overall_container=overall_container,
                variable_name_suffix=variable_name_suffix,
            )
        if isinstance(self, YAxisContainerMixIn):
            self._initialize_y_axis_container(
                overall_container=overall_container,
                variable_name_suffix=variable_name_suffix,
            )
        if isinstance(self, BorderContainerMixIn):
            self._initialize_border_container(
                overall_container=overall_container,
                variable_name_suffix=variable_name_suffix,
            )



"""This module is for the geometry-related values' validation
helper interfaces.
"""

from typing import Any


def validate_point_2d_type(*, point: Any, additional_err_msg: str = "") -> None:
    """
    Validate specified value's type is Point2D.

    Parameters
    ----------
    point : Point2D
        Point2D instance to be checked.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified value's type is not Point2D.
    """
    from apysc._geom.point2d import Point2D
    from apysc._validation import validation_common_utils

    if isinstance(point, Point2D):
        return
    err_msg: str = f"Specified value's type is not Point2D: {type(point)}"
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)



"""The class implementation for the outlined `rss_feed` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialRssFeedOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `rss_feed` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><circle cx="6.18" cy="17.82" r="2.18"/><path d="M4 4.44v2.83c7.03 0 12.73 5.7 12.73 12.73h2.83c0-8.59-6.97-15.56-15.56-15.56zm0 5.66v2.83c3.9 0 7.07 3.17 7.07 7.07h2.83c0-5.47-4.43-9.9-9.9-9.9z"/></svg>'  # noqa



"""Event validation implementation.

Mainly following interfaces are defined.

- validate_event
    Validate specified instance is Event.
"""

from typing import Any

from apysc._event.event import Event
from apysc._event.mouse_event_type import MouseEventType


def validate_event(*, e: Any, additional_err_msg: str = "") -> Event:
    """
    Validate whether a specified instance is the
    `Event` class or not.

    Parameters
    ----------
    e : Event
        Event instance to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        If a specified instance is not the `Event`
        class instance.

    Returns
    -------
    e : Event
        Event instance.
    """
    from apysc._validation import validation_common_utils

    if isinstance(e, Event):
        return e
    err_msg: str = f"Specified instance is not Event type: {type(e)}"
    err_msg = validation_common_utils.append_additional_err_msg(
        err_msg=err_msg, additional_err_msg=additional_err_msg
    )
    raise ValueError(err_msg)


def validate_event_type(*, mouse_event_type: Any) -> MouseEventType:
    """
    Validate whether specified value is MouseEventType one or not.

    Parameters
    ----------
    mouse_event_type : MouseEventType
        EventTMouseEventTypeype value to check.

    Returns
    -------
    mouse_event_type : MouseEventType
        MouseEventType value.
    """
    if isinstance(mouse_event_type, MouseEventType):
        return mouse_event_type
    raise ValueError(
        f"Specified value is not a MouseEventType: {type(mouse_event_type)}"
    )



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_round_rect.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics draw_round_rect interface": "# Graphics クラスの draw_round_rect インターフェイス",
    ##################################################
    "This page explains the `Graphics` class `draw_round_rect` method interface.": "このページでは`Graphics`クラスの`draw_round_rect`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "`draw_round_rect` interface draws vector rounded rectangle graphics.": "`draw_round_rect`インターフェイスは角丸の四角のベクターグラフィックスを描画します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "`draw_round_rect` interface has `x`, `y`, `width`, and `height` arguments. `x` and `y` are rectangle coordinates settings, and `width` and `height` determine rectangle size.": "`draw_round_rect`インターフェイスは`x`、`y`、`width`、`height`などの各引数を持っています。`x`と`y`引数は四角の座標を設定し、`width`と`height`は四角のサイズを決定します。",  # noqa
    ##################################################
    "This interface also has `ellipse_width` and `ellipse_height` arguments to set the round size to the rectangle corners.": "このインターフェイスはさらに角丸のサイズを設定するための`ellipse_width`と`ellipse_height`の引数を持っています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\n# Set 10-pixel ellipse size and draw the rectangle.\nsprite.graphics.draw_round_rect(\n    x=50, y=50, width=50, height=50, ellipse_width=10, ellipse_height=10\n)\n\n# Set 20-pixel ellipse size and draw the rectangle.\nsprite.graphics.draw_round_rect(\n    x=150, y=50, width=50, height=50, ellipse_width=20, ellipse_height=20\n)\n\n# Set 5-pixel ellipse width and 20-pixel ellipse height and\n# draw the rectangle.\nsprite.graphics.draw_round_rect(\n    x=250, y=50, width=50, height=50, ellipse_width=5, ellipse_height=20\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_round_rect_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\n# Set 10-pixel ellipse size and draw the rectangle.\nsprite.graphics.draw_round_rect(\n    x=50, y=50, width=50, height=50, ellipse_width=10, ellipse_height=10\n)\n\n# Set 20-pixel ellipse size and draw the rectangle.\nsprite.graphics.draw_round_rect(\n    x=150, y=50, width=50, height=50, ellipse_width=20, ellipse_height=20\n)\n\n# Set 5-pixel ellipse width and 20-pixel ellipse height and\n# draw the rectangle.\nsprite.graphics.draw_round_rect(\n    x=250, y=50, width=50, height=50, ellipse_width=5, ellipse_height=20\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_round_rect_basic_usage/")\n```',  # noqa
    ##################################################
    "## Return value": "## 返却値",
    ##################################################
    "`draw_round_rect` interface will return the `Rectangle` instance, the same as the `draw_rect` interface.": "`draw_round_rect`インターフェイスは`draw_rect`インターフェイスと同様に`Rectangle`クラスのインスタンスを返却します。",  # noqa
    ##################################################
    "The `Rectangle` instance has the `ellipse_width` attribute and `ellipse_height` to change the rectangle's round size.": "`Rectangle`クラスのインスタンスは四角の角丸のサイズを変更するための`ellipse_width`と`ellipse_height`属性を持っています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\nrectangle: ap.Rectangle = sprite.graphics.draw_round_rect(\n    x=50, y=50, width=50, height=50, ellipse_width=10, ellipse_height=10\n)\n\n# You can update the ellipse_width and ellipse_height\n# attributes dynamically.\nrectangle.ellipse_width = ap.Int(20)\n\nap.save_overall_html(dest_dir_path="graphics_draw_round_rect_return_value/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\nrectangle: ap.Rectangle = sprite.graphics.draw_round_rect(\n    x=50, y=50, width=50, height=50, ellipse_width=10, ellipse_height=10\n)\n\n# You can update the ellipse_width and ellipse_height\n# attributes dynamically.\nrectangle.ellipse_width = ap.Int(20)\n\nap.save_overall_html(dest_dir_path="graphics_draw_round_rect_return_value/")\n```',  # noqa
    ##################################################
    "## draw_round_rect API": "## draw_round_rect API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Draw a rounded rectangle vector graphics.<hr>": "角丸四角のベクターグラフィックスを描画します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x`: float or Number": "- `x`: float or Number",
    ##################################################
    "  - X-coordinate to start drawing.": "  - 描画を開始するX座標。",
    ##################################################
    "- `y`: float or Number": "- `y`: float or Number",
    ##################################################
    "  - Y-coordinate to start drawing.": "  - 描画を開始するY座標。",
    ##################################################
    "- `width`: Int or int": "- `width`: Int or int",
    ##################################################
    "  - Rectangle width.": "  - 四角の幅。",
    ##################################################
    "- `height`: Int or int": "- `height`: Int or int",
    ##################################################
    "  - Rectangle height.": "  - 四角の高さ。",
    ##################################################
    "- `ellipse_width`: Int or int": "- `ellipse_width`: Int or int",
    ##################################################
    "  - Ellipse width of the rectangle corner.": "  - 四角の角丸の幅。",
    ##################################################
    "- `ellipse_height`: Int or int": "- `ellipse_height`: Int or int",
    ##################################################
    "  - Ellipse height of the rectangle corner.": "  - 四角の角丸の高さ。",
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `rectangle`: Rectangle": "- `rectangle`: Rectangle",
    ##################################################
    "  - Created rectangle.": "  - 生成された四角。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> round_rect: ap.Rectangle = sprite.graphics.draw_round_rect(\n...     x=50, y=50, width=50, height=50, ellipse_width=10, ellipse_height=15\n... )\n>>> round_rect.ellipse_width\nInt(10)\n\n>>> round_rect.ellipse_height\nInt(15)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> round_rect: ap.Rectangle = sprite.graphics.draw_round_rect(\n...     x=50, y=50, width=50, height=50, ellipse_width=10, ellipse_height=15\n... )\n>>> round_rect.ellipse_width\nInt(10)\n\n>>> round_rect.ellipse_height\nInt(15)\n```',  # noqa
}



"""The class implementation for the outlined `policy` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPolicyOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `policy` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g/><g><path d="M12,1L3,5v6c0,5.55,3.84,10.74,9,12c5.16-1.26,9-6.45,9-12V5L12,1z M19,11c0,1.85-0.51,3.65-1.38,5.21l-1.45-1.45 c1.29-1.94,1.07-4.58-0.64-6.29c-1.95-1.95-5.12-1.95-7.07,0c-1.95,1.95-1.95,5.12,0,7.07c1.71,1.71,4.35,1.92,6.29,0.64 l1.72,1.72c-1.19,1.42-2.73,2.51-4.47,3.04C7.98,19.69,5,15.52,5,11V6.3l7-3.11l7,3.11V11z M12,15c-1.66,0-3-1.34-3-3s1.34-3,3-3 s3,1.34,3,3S13.66,15,12,15z"/></g></g></svg>'  # noqa



"""The class implementation for the `api` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialApiIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `api` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M14,12l-2,2l-2-2l2-2L14,12z M12,6l2.12,2.12l2.5-2.5L12,1L7.38,5.62l2.5,2.5L12,6z M6,12l2.12-2.12l-2.5-2.5L1,12 l4.62,4.62l2.5-2.5L6,12z M18,12l-2.12,2.12l2.5,2.5L23,12l-4.62-4.62l-2.5,2.5L18,12z M12,18l-2.12-2.12l-2.5,2.5L12,23l4.62-4.62 l-2.5-2.5L12,18z"/></g></svg>'  # noqa



"""The class implementation for the `link_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLinkOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `link_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M17 7h-4v1.9h4c1.71 0 3.1 1.39 3.1 3.1 0 1.43-.98 2.63-2.31 2.98l1.46 1.46C20.88 15.61 22 13.95 22 12c0-2.76-2.24-5-5-5zm-1 4h-2.19l2 2H16zM2 4.27l3.11 3.11C3.29 8.12 2 9.91 2 12c0 2.76 2.24 5 5 5h4v-1.9H7c-1.71 0-3.1-1.39-3.1-3.1 0-1.59 1.21-2.9 2.76-3.07L8.73 11H8v2h2.73L13 15.27V17h1.73l4.01 4L20 19.74 3.27 3 2 4.27z"/><path d="M0 24V0" fill="none"/></svg>'  # noqa



"""The class implementation for the `card_giftcard` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCardGiftcardIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `card_giftcard` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M20 6h-2.18c.11-.31.18-.65.18-1 0-1.66-1.34-3-3-3-1.05 0-1.96.54-2.5 1.35l-.5.67-.5-.68C10.96 2.54 10.05 2 9 2 7.34 2 6 3.34 6 5c0 .35.07.69.18 1H4c-1.11 0-1.99.89-1.99 2L2 19c0 1.11.89 2 2 2h16c1.11 0 2-.89 2-2V8c0-1.11-.89-2-2-2zm-5-2c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zM9 4c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm11 15H4v-2h16v2zm0-5H4V8h5.08L7 10.83 8.62 12 11 8.76l1-1.36 1 1.36L15.38 12 17 10.83 14.92 8H20v6z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_round_dotted_line.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics draw_round_dotted_line interface": "# Graphics クラスの draw_round_dotted_line インターフェイス",  # noqa
    ##################################################
    "This page explains the `Graphics` class `draw_round_dotted_line` method interface.": "このページでは`Graphics`クラスの`draw_round_dotted_line`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "`draw_round_dotted_line` interface draws the simple straight round dotted-line graphics. This interface ignores `dot_setting`\\, `dash_setting`\\, `round_dot_setting`\\, `dash_dot_setting`\\, and `cap` settings (this interface is using round cap setting so cap setting will also be ignored).": "`draw_round_dotted_line`インターフェイスはシンプルな丸ドットの直線のグラフィックスを描画します。このインターフェイスは`dot_setting`、`dash_setting`、`round_dot_setting`、`dash_dot_setting`、`cap`の各設定を無視します（このインターフェイスでは丸の`cap`設定を内部で使っているため`cap`設定は無視されます）。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "`draw_round_dotted_line` interface has basic coordinates arguments of `x_start`, `y_start`, `x_end` and `y_end`. That also has `round_size` and `space_size` arguments to determine the round style (line round size and the space size between each round).": "`draw_round_dotted_line`インターフェイスは基本的な座標指定の引数として`x_start`、`y_start`、`x_end`、`y_end`の引数を持っています。それらに加えて丸のサイズの`round_size`と丸の間のスペースを決定する`space_size`引数の指定が必要になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=130,\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 5-pixel round size and draw the line.\nsprite.graphics.line_style(color=ap.Color("#0af"))\nsprite.graphics.draw_round_dotted_line(\n    x_start=50, y_start=50, x_end=200, y_end=50, round_size=5, space_size=5\n)\n\n# Set 10-pixel round size and draw the line.\nsprite.graphics.draw_round_dotted_line(\n    x_start=50, y_start=80, x_end=200, y_end=80, round_size=10, space_size=5\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_round_dotted_line_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=130,\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 5-pixel round size and draw the line.\nsprite.graphics.line_style(color=ap.Color("#0af"))\nsprite.graphics.draw_round_dotted_line(\n    x_start=50, y_start=50, x_end=200, y_end=50, round_size=5, space_size=5\n)\n\n# Set 10-pixel round size and draw the line.\nsprite.graphics.draw_round_dotted_line(\n    x_start=50, y_start=80, x_end=200, y_end=80, round_size=10, space_size=5\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_round_dotted_line_basic_usage/")\n```',  # noqa
    ##################################################
    "## Notes": "## 特記事項",
    ##################################################
    "Since this interface uses the round cap setting, the line length becomes longer by the size of the cap.": "このインターフェイスは丸のcap（線の端のスタイル）設定を使用しているため、線の長さはcapのサイズに応じて大きくなります。",  # noqa
    ##################################################
    "If you want to align the left line position with other lines, subtract half-round size from the `x_start` argument.": "もしも線の左端を他の線と合わせたい場合には丸のサイズの半分を`x_start`の引数から減算してください。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=270,\n    stage_height=130,\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 5-pixel round size and draw the line.\nsprite.graphics.line_style(color=ap.Color("#0af"))\nsprite.graphics.draw_round_dotted_line(\n    x_start=50, y_start=50, x_end=220, y_end=50, round_size=10, space_size=5\n)\n\n# Set 45-pixel (50 - half-round size) to x_start argument\n# and draw the normal line.\nsprite.graphics.draw_line(x_start=45, y_start=80, x_end=225, y_end=80)\n\nap.save_overall_html(dest_dir_path="graphics_draw_round_dotted_line_notes/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=270,\n    stage_height=130,\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 5-pixel round size and draw the line.\nsprite.graphics.line_style(color=ap.Color("#0af"))\nsprite.graphics.draw_round_dotted_line(\n    x_start=50, y_start=50, x_end=220, y_end=50, round_size=10, space_size=5\n)\n\n# Set 45-pixel (50 - half-round size) to x_start argument\n# and draw the normal line.\nsprite.graphics.draw_line(x_start=45, y_start=80, x_end=225, y_end=80)\n\nap.save_overall_html(dest_dir_path="graphics_draw_round_dotted_line_notes/")\n```',  # noqa
    ##################################################
    "## draw_round_dotted_line API": "## draw_round_dotted_line API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Draw a round-dotted line vector graphics.<hr>": "丸ドットの直線のベクターグラフィックスを描画します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x_start`: float or Number": "- `x_start`: float or Number",
    ##################################################
    "  - Line start x-coordinate.": "  - 線の開始位置のX座標。",
    ##################################################
    "- `y_start`: float or Number": "- `y_start`: float or Number",
    ##################################################
    "  - Line start y-coordinate.": "  - 線の開始位置のY座標。",
    ##################################################
    "- `x_end`: float or Number": "- `x_end`: float or Number",
    ##################################################
    "  - Line end x-coordinate.": "  - 線の終了位置のX座標。",
    ##################################################
    "- `y_end`: float or Number": "- `y_end`: float or Number",
    ##################################################
    "  - Line end y-coordinate.": "  - 線の終了位置のY座標。",
    ##################################################
    "- `round_size`: Int or int": "- `round_size`: Int or int",
    ##################################################
    "  - Dot round size.": "  - 丸ドットのサイズ。",
    ##################################################
    "- `space_size`: Int or int": "- `space_size`: Int or int",
    ##################################################
    "  - Blank space size between dots.": "  - ドット間の空白のスペースのサイズ。",
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line`: Line": "- `line`: Line",
    ##################################################
    "  - Created line graphics instance.": "  - 生成された線のグラフィックスのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "This interface ignores line settings, like the `LineDotSetting`, except `LineRoundDotSetting`.<hr>": "このインターフェイスは`LineRoundDotSetting`を除いて`LineDotSetting`などの設定を無視します。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_round_dotted_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50, round_size=6, space_size=3\n... )\n>>> line.line_color\nColor("#ffffff")\n\n>>> line.line_round_dot_setting.round_size\nInt(6)\n\n>>> line.line_round_dot_setting.space_size\nInt(3)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_round_dotted_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50, round_size=6, space_size=3\n... )\n>>> line.line_color\nColor("#ffffff")\n\n>>> line.line_round_dot_setting.round_size\nInt(6)\n\n>>> line.line_round_dot_setting.space_size\nInt(3)\n```',  # noqa
}



"""Class implementation for the animation_scale_y_from_point
interface.
"""

from typing import Union

from typing_extensions import final

from apysc._animation.animation_mixins import AnimationMixIns
from apysc._animation.animation_scale_y_from_point import AnimationScaleYFromPoint
from apysc._animation.easing import Easing
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class AnimationScaleYFromPointMixIn(AnimationMixIns):
    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=3, optional=False)
    @arg_validation_decos.num_is_gt_zero(arg_position_index=3, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=4, optional=False)
    @arg_validation_decos.is_easing(arg_position_index=5)
    def animation_scale_y_from_point(
        self,
        *,
        scale_y_from_point: Union[float, Number],
        y: Union[float, Number],
        duration: Union[int, Int] = 3000,
        delay: Union[int, Int] = 0,
        easing: Easing = Easing.LINEAR
    ) -> AnimationScaleYFromPoint:
        """
        Set the scale-y from the given point animation setting.

        Notes
        -----
        To start this animation, you need to call the `start` method of
        the returned instance.

        Parameters
        ----------
        scale_y_from_point : float or Number
            The final scale-y from the given point of the animation.
        y : float or Number
            Y-coordinate.
        duration : Int or int, default 3000
            Milliseconds before an animation ends.
        delay : Int or int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.

        Returns
        -------
        animation_scale_y_from_point : AnimationScaleYFromPoint
            Created animation setting instance.

        References
        ----------
        - animation_scale_from_point interfaces
            - https://simon-ritchie.github.io/apysc/en/animation_scale_x_and_y_from_point.html  # noqa
        - Animation interfaces duration setting
            - https://simon-ritchie.github.io/apysc/en/animation_duration.html
        - Animation interfaces delay setting
            - https://simon-ritchie.github.io/apysc/en/animation_delay.html
        - Each animation interface return value
            - https://simon-ritchie.github.io/apysc/en/animation_return_value.html  # noqa
        - Sequential animation setting
            - https://simon-ritchie.github.io/apysc/en/sequential_animation.html
        - animation_parallel interface
            - https://simon-ritchie.github.io/apysc/en/animation_parallel.html
        - Easing enum
            - https://simon-ritchie.github.io/apysc/en/easing_enum.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> _ = rectangle.animation_scale_y_from_point(
        ...     scale_y_from_point=0.5,
        ...     y=ap.Number(100),
        ...     duration=1500,
        ...     easing=ap.Easing.EASE_OUT_QUINT,
        ... ).start()
        """
        animation_scale_y_from_point: AnimationScaleYFromPoint = (
            AnimationScaleYFromPoint(
                target=self,
                scale_y_from_point=scale_y_from_point,
                y=y,
                duration=duration,
                delay=delay,
                easing=easing,
            )
        )
        return animation_scale_y_from_point



"""The class implementation for the `edit_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialEditOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `edit_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><title>ic_edit_off_24px</title><path d="M0 0h24v24H0zm0 0h24v24H0z" fill="none"/><path d="M12.126 8.125l1.937-1.937 3.747 3.747-1.937 1.938zM20.71 5.63l-2.34-2.34a1 1 0 0 0-1.41 0l-1.83 1.83 3.75 3.75L20.71 7a1 1 0 0 0 0-1.37zM2 5l6.63 6.63L3 17.25V21h3.75l5.63-5.62L18 21l2-2L4 3 2 5z"/></svg>'  # noqa



"""The utility module for the documents toctree.
"""

import os
import re
from datetime import datetime
from logging import Logger
from typing import List
from typing import Match
from typing import Optional
from typing import Pattern
from typing import Tuple

from apysc._console import loggers
from apysc._lint_and_doc.docs_lang import Lang

_TOCTREE_DEFINED_EN_FILE_NAMES: List[str] = [
    "index",
]

logger: Logger = loggers.get_info_logger()


def get_toctree_file_names() -> List[str]:
    """
    Get toctree file names from the toctree defined files.

    Returns
    -------
    toctree_file_names : List[str]
        Result toctree file names
        (e.g., ["what_apysc_can_do.md", "quick_start.md", ...]).
    """
    toctree_file_names: List[str] = []
    for toctree_defined_en_file_name in _TOCTREE_DEFINED_EN_FILE_NAMES:
        extracted_toctree_file_names: List[str] = _extract_toctree_file_names_from_file(
            toctree_defined_en_file_name=toctree_defined_en_file_name,
        )
        toctree_file_names.extend(extracted_toctree_file_names)
    return toctree_file_names


def _extract_toctree_file_names_from_file(
    *, toctree_defined_en_file_name: str
) -> List[str]:
    """
    Extract toctree file names from a specified file.

    Parameters
    ----------
    toctree_defined_en_file_name : str
        An English document file name that contains the toctree definition.

    Returns
    -------
    toctree_file_names : List[str]
        Extracted toctree file names.
    """
    from apysc._file import file_util

    file_str: str = file_util.read_txt(
        file_path=f"./docs_src/source/{toctree_defined_en_file_name}.md"
    )
    lines: List[str] = file_str.splitlines()
    toctree_file_names: List[str] = []
    is_toctree_range_lines: bool = False
    for line in lines:
        if "```{toctree}" in line:
            is_toctree_range_lines = True
            continue
        if "maxdepth" in line:
            continue
        if is_toctree_range_lines and "```" in line:
            is_toctree_range_lines = False
            continue
        if not is_toctree_range_lines:
            continue
        toctree_file_names.append(f"{line.strip()}.md")
    return toctree_file_names


_LINK_PREV_PATTERN: Pattern = re.compile(
    pattern=r'\<link rel="prev" title=".+?" href="(.+?)\.html"'
)
_LINK_NEXT_PATTERN: Pattern = re.compile(
    pattern=r'\<link rel="next" title=".+?" href="(.+?)\.html"'
)


def get_doc_prev_and_next_md_file_names(*, md_doc_file_name: str) -> Tuple[str, str]:
    """
    Get specified markdown file's previous and next markdown file names.

    Parameters
    ----------
    md_doc_file_name : str
        Target markdown file name (e.g., sprite.md).

    Returns
    -------
    prev_md_doc_file_name : str
        A previous markdown file name.
    next_md_doc_file_name : str
        A next markdown file name.
    """
    from apysc._file import file_util

    html_file_path: str = f"./docs/en/{md_doc_file_name.replace('.md', '.html', 1)}"
    if not os.path.exists(html_file_path):
        return "", ""
    prev_md_doc_file_name: str = ""
    next_md_doc_file_name: str = ""
    html_str: str = file_util.read_txt(file_path=html_file_path)
    lines: List[str] = html_str.splitlines()
    for line in lines:
        if prev_md_doc_file_name == "":
            match: Optional[Match] = _LINK_PREV_PATTERN.search(string=line)
            if match is not None:
                prev_md_doc_file_name = f"{match.group(1)}.md"
                continue
        if next_md_doc_file_name == "":
            match = _LINK_NEXT_PATTERN.search(string=line)
            if match is not None:
                next_md_doc_file_name = f"{match.group(1)}.md"
                continue
    return prev_md_doc_file_name, next_md_doc_file_name


def update_docs_prev_and_next_page_modified_time() -> None:
    """
    Update documents next and previous pages' modified time.
    """
    toctree_file_names: List[str] = get_toctree_file_names()
    prev_md_doc_file_name: str
    next_md_doc_file_name: str
    for i, toctree_file_name in enumerate(toctree_file_names):
        (
            prev_md_doc_file_name,
            next_md_doc_file_name,
        ) = get_doc_prev_and_next_md_file_names(md_doc_file_name=toctree_file_name)
        expected_prev_md_file_name: str = _get_expected_prev_md_file_name(
            toctree_file_names=toctree_file_names,
            i=i,
        )
        expected_next_md_file_name: str = _get_expected_next_md_file_name(
            toctree_file_names=toctree_file_names,
            i=i,
        )
        _update_adjacent_doc_modified_time_if_toctree_updated(
            adjacent_doc_file_name=prev_md_doc_file_name,
            expected_md_file_name=expected_prev_md_file_name,
        )
        _update_adjacent_doc_modified_time_if_toctree_updated(
            adjacent_doc_file_name=next_md_doc_file_name,
            expected_md_file_name=expected_next_md_file_name,
        )


def _update_adjacent_doc_modified_time_if_toctree_updated(
    *,
    adjacent_doc_file_name: str,
    expected_md_file_name: str,
) -> bool:
    """
    Update a specified adjacent document's modified time
    if a related toctree differs from an expected file name.

    Parameters
    ----------
    adjacent_doc_file_name : str
        An adjacent document file name.
    expected_md_file_name : str
        An expected document file name.

    Returns
    -------
    is_updated : bool
        A boolean indicates whether this function updated an adjacent document's
        modified time or not.
    """
    if adjacent_doc_file_name == expected_md_file_name:
        return False
    if adjacent_doc_file_name.strip() == "":
        return False
    now_unix_time: float = datetime.now().timestamp()
    file_path: str = f"./docs_src/source/{adjacent_doc_file_name}"
    if not os.path.exists(file_path):
        return False
    os.utime(file_path, (now_unix_time, now_unix_time))
    logger.info(msg=f"Document's modified time is updated: {adjacent_doc_file_name}")
    for lang in Lang:
        if lang == Lang.EN:
            continue
        other_lang_doc_file_path: str = (
            f"./docs_src/source/{lang.value}_{adjacent_doc_file_name}"
        )
        if not os.path.exists(other_lang_doc_file_path):
            continue
        os.utime(other_lang_doc_file_path, (now_unix_time, now_unix_time))
        logger.info(
            msg=f"Document's modified time is updated: {other_lang_doc_file_path}"
        )
    return True


def _get_expected_next_md_file_name(*, toctree_file_names: List[str], i: int) -> str:
    """
    Get an expected next markdown file name from a specified index and list.

    Parameters
    ----------
    toctree_file_names : List[str]
        A list of markdown file names.
    i : int
        Target list index.

    Returns
    -------
    expected_next_md_file_name : str
        An expected next markdown file name.
    """
    if i == len(toctree_file_names) - 1:
        return ""
    expected_next_md_file_name: str = toctree_file_names[i + 1]
    return expected_next_md_file_name


def _get_expected_prev_md_file_name(*, toctree_file_names: List[str], i: int) -> str:
    """
    Get an expected previous markdown file name from a specified index and list.

    Parameters
    ----------
    toctree_file_names : List[str]
        A list of markdown file names.
    i : int
        Target list index.

    Returns
    -------
    expected_prev_md_file_name : str
        An expected previous markdown file name.
    """
    if i == 0:
        return ""
    expected_prev_md_file_name: str = toctree_file_names[i - 1]
    return expected_prev_md_file_name



"""The mix-in class implementation for the `SvgText`'s `_set_bold` method.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean


class SvgTextSetBoldMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_bold(self, *, bold: Optional[Union[bool, Boolean]]) -> None:
        """
        Set a bold style setting.

        Parameters
        ----------
        bold : Optional[Union[bool, Boolean]]
            A boolean, whether a text is a bold style or not (normal).
            If a specified value is `None`, this interface ignores the setting.
        """
        from apysc._display.svg_text_bold_mixin import SvgTextBoldMixIn
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        if bold is None:
            return

        if not isinstance(self, SvgTextBoldMixIn):
            raise TypeError(
                f"This method is only supported an {SvgTextBoldMixIn.__name__} "
                f"instance: {type(self).__name__}"
            )

        if isinstance(bold, bool):
            suffix: str = get_attr_or_variable_name_suffix(
                instance=self,
                value_identifier="bold",
            )
            bold_: Boolean = Boolean(bold, variable_name_suffix=suffix)
        else:
            bold_ = bold
        self.bold = bold_



"""The class implementation for the outlined `voice_over_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialVoiceOverOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `voice_over_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M16.76 5.36l-1.68 1.69c.8 1.13.83 2.58.09 3.74l1.7 1.7c1.9-2.02 1.87-4.98-.11-7.13zM20.07 2l-1.63 1.63c2.72 2.97 2.76 7.39.14 10.56l1.64 1.64c3.74-3.89 3.71-9.84-.15-13.83zM9.43 5.04l3.53 3.53c-.2-1.86-1.67-3.33-3.53-3.53zM4.41 2.86L3 4.27l2.62 2.62C5.23 7.5 5 8.22 5 9c0 2.21 1.79 4 4 4 .78 0 1.5-.23 2.11-.62l4.4 4.4C13.74 15.6 10.78 15 9 15c-2.67 0-8 1.34-8 4v2h16v-2c0-.37-.11-.7-.29-1.02L19.73 21l1.41-1.41L4.41 2.86zM3 19c.22-.72 3.31-2 6-2 2.7 0 5.8 1.29 6 2H3zm6-8c-1.1 0-2-.9-2-2 0-.22.04-.42.11-.62l2.51 2.51c-.2.07-.4.11-.62.11z"/></svg>'  # noqa



"""The mix-in class implementation for `border_container`-related interfaces.
"""

from typing_extensions import final

from apysc._display.sprite import Sprite
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class BorderContainerMixIn:
    _border_container: Sprite

    @final
    @arg_validation_decos.is_display_object_container(
        arg_position_index=1, optional=False
    )
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _initialize_border_container(
        self,
        *,
        overall_container: Sprite,
        variable_name_suffix: str,
    ) -> None:
        """
        Initialize a border container.

        Parameters
        ----------
        overall_container : Sprite
            An overall container instance.
        variable_name_suffix : str
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        self._border_container = Sprite(variable_name_suffix=variable_name_suffix)
        overall_container.add_child(self._border_container)



"""The class implementation for the `nightlight_round` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialNightlightRoundIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `nightlight_round` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12.01 12c0-3.57 2.2-6.62 5.31-7.87.89-.36.75-1.69-.19-1.9-1.1-.24-2.27-.3-3.48-.14-4.51.6-8.12 4.31-8.59 8.83C4.44 16.93 9.13 22 15.01 22c.73 0 1.43-.08 2.12-.23.95-.21 1.1-1.53.2-1.9-3.22-1.29-5.33-4.41-5.32-7.87z"/></svg>'  # noqa



"""The class implementation for the `contacts` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialContactsIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `contacts` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0zm0 0h24v24H0zm0 0h24v24H0z" fill="none"/><path d="M20 0H4v2h16V0zM4 24h16v-2H4v2zM20 4H4c-1.1 0-2 .9-2 2v12c0 1.1.9 2 2 2h16c1.1 0 2-.9 2-2V6c0-1.1-.9-2-2-2zm-8 2.75c1.24 0 2.25 1.01 2.25 2.25s-1.01 2.25-2.25 2.25S9.75 10.24 9.75 9 10.76 6.75 12 6.75zM17 17H7v-1.5c0-1.67 3.33-2.5 5-2.5s5 .83 5 2.5V17z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: continue.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Continue class": "# Continue クラス",
    ##################################################
    "This page explains the `Continue` class.": "このページでは`Continue`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page (apysc uses the `Continue` class for the same reason):": "このページを読み進める前に以下のページを確認しておくと役に立つかもしれません（apyscでは`Continue`クラスを同様の利用で使用しています）。",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What is the Continue class?": "## Continue クラスの概要",
    ##################################################
    "The `with ap.ForArrayIndices` block uses the `Continue` class to skip a current loop iteration (in JavaScript). It behaves like the Python built-in `continue` keyword.": "`with For`のブロックではJavaScript上での特定のループをスキップするために`Continue`クラスが使用されます。このインターフェイスはPythonビルトインの`continue`キーワードのように動作します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `Continue` class can only be used in the `with ap.ForArrayIndices` (or other loop class) block, as follows:": "`Continue`クラスは以下のコード例のように`with For`（もしくは他のループクラス）のブロックでのみ使用することができます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\narr: ap.Array = ap.Array(range(2))\nwith ap.ForArrayIndices(arr) as i:\n    condition: ap.Boolean = i == 0\n    with ap.If(condition):\n        sprite.graphics.begin_fill(color=ap.Color("#0af"))\n        sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n        ap.Continue()\n\n    sprite.graphics.begin_fill(color=ap.Color("#f0a"))\n    sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.save_overall_html(dest_dir_path="continue_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\n\narr: ap.Array = ap.Array(range(2))\nwith ap.ForArrayIndices(arr) as i:\n    condition: ap.Boolean = i == 0\n    with ap.If(condition):\n        sprite.graphics.begin_fill(color=ap.Color("#0af"))\n        sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n        ap.Continue()\n\n    sprite.graphics.begin_fill(color=ap.Color("#f0a"))\n    sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.save_overall_html(dest_dir_path="continue_basic_usage/")\n```',  # noqa
    ##################################################
    "If you use the `Continue` class in the out of the `with ap.ForArrayIndices` block, then an exception is raised:": "もし`Continue`クラスを`with For`ブロック外で使用した場合はエラーになります:",  # noqa
    ##################################################
    "```py\nimport apysc as ap\n\nap.Continue()\n```": "```py\nimport apysc as ap\n\nap.Continue()\n```",  # noqa
    ##################################################
    "```\nException: The `Continue` class can be instantiated in the with loop statement, for example, after the `with ap.ForArrayIndices(...):` statement.\n```": "```\nException: The `Continue` class can be instantiated in the with loop statement, for example, after the `with ap.ForArrayIndices(...):` statement.\n```",  # noqa
    ##################################################
    "## Continue API": "## Continue API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "The loop continue expression class.<hr>": "ループのcontinueの表現を扱うためのクラスです。<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "This class can be instantiated in the with loop statement, for example, after the `with ap.ForArrayIndices(...):` statement.<hr>": "このクラスはwithステートメントのループ内でのみインスタンス化することができます。例えば`with ap.ForArrayIndices(...)ステート内が該当します。`<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array(range(3))\n>>> with ap.ForArrayIndices(arr) as i:\n...     with ap.If(i == 1):\n...         _ = ap.Continue()\n...\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array(range(3))\n>>> with ap.ForArrayIndices(arr) as i:\n...     with ap.If(i == 1):\n...         _ = ap.Continue()\n...\n```",  # noqa
}



"""The class implementation for the outlined `mediation` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMediationOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `mediation` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M18,16l4-4l-4-4v3h-5.06c-0.34-3.1-2.26-5.72-4.94-7.05C7.96,2.31,6.64,1,5,1C3.34,1,2,2.34,2,4s1.34,3,3,3 c0.95,0,1.78-0.45,2.33-1.14C9.23,6.9,10.6,8.77,10.92,11h-3.1C7.4,9.84,6.3,9,5,9c-1.66,0-3,1.34-3,3s1.34,3,3,3 c1.3,0,2.4-0.84,2.82-2h3.1c-0.32,2.23-1.69,4.1-3.58,5.14C6.78,17.45,5.95,17,5,17c-1.66,0-3,1.34-3,3s1.34,3,3,3 c1.64,0,2.96-1.31,2.99-2.95c2.68-1.33,4.6-3.95,4.94-7.05H18V16z"/></g></svg>'  # noqa



"""The class implementation for the outlined `domain_disabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialDomainDisabledOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `domain_disabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M1.41 1.69L0 3.1l2 2V21h15.9l3 3 1.41-1.41-20.9-20.9zM6 19H4v-2h2v2zm0-4H4v-2h2v2zm-2-4V9h2v2H4zm6 8H8v-2h2v2zm-2-4v-2h2v2H8zm4 4v-2h1.9l2 2H12zM8 5h2v2h-.45L12 9.45V9h8v8.45l2 2V7H12V3H5.55L8 5.45zm8 6h2v2h-2z"/></svg>'  # noqa



"""Class implementation for the animation_rotation_around_center
mix-in.
"""

from typing import Union

from typing_extensions import final

from apysc._animation.animation_mixins import AnimationMixIns
from apysc._animation.animation_rotation_around_center import (
    AnimationRotationAroundCenter,
)
from apysc._animation.easing import Easing
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class AnimationRotationAroundCenterMixIn(AnimationMixIns):
    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=2, optional=False)
    @arg_validation_decos.num_is_gt_zero(arg_position_index=2, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=3, optional=False)
    @arg_validation_decos.is_easing(arg_position_index=4)
    def animation_rotation_around_center(
        self,
        *,
        rotation_around_center: Union[int, Int],
        duration: Union[int, Int] = 3000,
        delay: Union[int, Int] = 0,
        easing: Easing = Easing.LINEAR
    ) -> AnimationRotationAroundCenter:
        """
        Set the rotation around the center animation setting.

        Notes
        -----
        To start this animation, you need to call the `start` method of
        the returned instance.

        Parameters
        ----------
        rotation_around_center : Int or int
            The final rotation of the animation.
        duration : Int or int, default 3000
            Milliseconds before an animation ends.
        delay : Int or int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.

        Returns
        -------
        animation_rotation_around_center : AnimationRotationAroundCenter
            Created animation setting instance.

        References
        ----------
        - animation_rotation_around_center interface
            - https://simon-ritchie.github.io/apysc/en/animation_rotation_around_center.html  # noqa
        - Animation interfaces duration setting
            - https://simon-ritchie.github.io/apysc/en/animation_duration.html
        - Animation interfaces delay setting
            - https://simon-ritchie.github.io/apysc/en/animation_delay.html
        - Each animation interface return value
            - https://simon-ritchie.github.io/apysc/en/animation_return_value.html  # noqa
        - Sequential animation setting
            - https://simon-ritchie.github.io/apysc/en/sequential_animation.html
        - animation_parallel interface
            - https://simon-ritchie.github.io/apysc/en/animation_parallel.html
        - Easing enum
            - https://simon-ritchie.github.io/apysc/en/easing_enum.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> _ = rectangle.animation_rotation_around_center(
        ...     rotation_around_center=90,
        ...     duration=1500,
        ...     easing=ap.Easing.EASE_OUT_QUINT,
        ... ).start()
        """
        animation_rotation_around_center: AnimationRotationAroundCenter = (
            AnimationRotationAroundCenter(
                target=self,
                rotation_around_center=rotation_around_center,
                duration=duration,
                delay=delay,
                easing=easing,
            )
        )
        return animation_rotation_around_center



"""The mix-in class implementation for the `String`'s
`apply_max_num_of_decimal_places` method.
"""

import re
from typing import TYPE_CHECKING
from typing import Any
from typing import Match
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._type.int import Int
    from apysc._type.string import String


class StringApplyMaxNumOfDecimalPlacesMixIn:
    @final
    @arg_validation_decos.is_string(arg_position_index=0, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def apply_max_num_of_decimal_places(
        self,
        *,
        max_num_of_decimal_places: Union[int, "Int"],
        variable_name_suffix: str = "",
    ) -> "String":
        """
        Apply a maximum number of decimal places limit to this string.

        Parameters
        ----------
        max_num_of_decimal_places : Union[int, Int]
            A maximum number of decimal places.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        string : String
            An applied string.

        References
        ----------
        - String class apply_max_num_of_decimal_places interface
            - https://simon-ritchie.github.io/apysc/en/string_apply_max_num_of_decimal_places.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> string = ap.String("123.456")
        >>> string = string.apply_max_num_of_decimal_places(max_num_of_decimal_places=1)
        >>> ap.assert_equal(string, "123.4")
        """
        from apysc._expression import expression_data_util
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._type.int import Int
        from apysc._type.string import String
        from apysc._validation import variable_name_validation

        if not isinstance(max_num_of_decimal_places, Int):
            max_num_of_decimal_places_: Int = Int(
                max_num_of_decimal_places, variable_name_suffix=variable_name_suffix
            )
        else:
            max_num_of_decimal_places_ = max_num_of_decimal_places
        string: String = String("", variable_name_suffix=variable_name_suffix)
        string._value = _get_py_str(
            max_num_of_decimal_places=max_num_of_decimal_places_,
            self_str=self,
        )

        self_variable_name: str = (
            variable_name_validation.validate_variable_name_mixin_type(
                instance=self
            ).variable_name
        )
        matched_variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.MATCHED,
        )
        expression: str = (
            f"var {matched_variable_name} = {self_variable_name}"
            f'.match(new RegExp("^(\\\\d+\\\\.\\\\d{{"'
            f' + {max_num_of_decimal_places_.variable_name} + "}})\\\\d*$"));'
            f"\nif ({matched_variable_name}) {{"
            f"\n  {string.variable_name} = {matched_variable_name}[1];"
            "\n} else {"
            f"\n  {string.variable_name} = {self_variable_name};"
            "\n}"
        )
        expression_data_util.append_js_expression(expression=expression)

        return string


@add_debug_info_setting(module_name=__name__)
@arg_validation_decos.is_apysc_integer(arg_position_index=0)
def _get_py_str(
    *,
    max_num_of_decimal_places: "Int",
    self_str: Any,
) -> str:
    """
    Get a Python value string.

    Parameters
    ----------
    max_num_of_decimal_places : Int
        A maximum number of decimal places.
    self_str : Any
        A self `String` instance.

    Returns
    -------
    py_str : str
        A Python value string.
    """
    from apysc._type.string import String

    py_str: str = ""
    if isinstance(self_str, String):
        py_str = self_str._value
    match_: Optional[Match] = re.match(
        pattern=rf"^(\d+\.\d{{{max_num_of_decimal_places._value}}})\d*$",
        string=py_str,
    )
    if match_ is not None:
        py_str = match_.group(1)
    return py_str
